import { Component, OnInit, Input, Output, EventEmitter, ViewContainerRef, ViewChild, OnDestroy, AfterViewInit } from '@angular/core';
import { Subject, Subscription } from 'rxjs';
import { NzNotificationService } from 'ng-zorro-antd/notification';
import { Util } from '@global/util/util';
import { NzModalService } from 'ng-zorro-antd/modal';
import { FormButtonModel, FormPageModel, FormItemModel, FormDRUIPartModel, FormPartModel, FormGroupPanelModel, FormIFrameModel, FormRowItemModel, FormTabPageModel, FormTabPanelModel, FormUserControlModel } from '@global/model/form-detail';
import { debounceTime, distinctUntilChanged } from 'rxjs/operators';

@Component({
	selector: 'main-form',
	templateUrl: './main.html',
	styleUrls: ['./main.less'],
})
export class EditForm implements OnInit {

	constructor(private $util: Util) {
		
	}

    /**
     * 数据对象
     *
     * @type {*}
     * @memberof EditForm
     */
    @Input()
    set formData(val: any) {
        if (val) {
            this.data = JSON.parse(JSON.stringify(val));
            this.formLogic('', null, null);
        }
    }

    /**
     * 表单数据对象
     *
     * @type {*}
     * @memberof EditForm
     */
    public data: any = {};

	/**
	 * 表单数据变化
	 *
	 * @type {Subject<any>}
	 * @memberof EditForm
	 */
	public formDataChange: Subject<any> = new Subject<any>();

	/**
	 * 表单数据变化抛出事件
	 *
	 * @type {EventEmitter<any>}
	 * @memberof EditForm
	 */
	@Output() dataChangeEvent: EventEmitter<any> = new EventEmitter<any>();

    /**
     * 代码表模型
     *
     * @type {*}
     * @memberof Main
     */
    public codelistModel: any = {
    };

    /**
     * 属性值规则
     *
     * @type {*}
     * @memberof Main
     */
    public rules: any = {
        formitem: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem1: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem2: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem3: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem4: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem5: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem6: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem7: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem8: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem9: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem10: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem11: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem12: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem13: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
        formitem14: [
            { type: 'string', message: '表单项 值必须为字符串类型' },
            { required: false, type: 'string', message: '表单项 值不能为空' },
        ],
    }

    /**
     * 详情模型集合
     *
     * @type {*}
     * @memberof Main
     */
    public detailsModel: any = {
        formitem: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem1: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem1', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        tabpage1: new FormTabPageModel({ caption: '分页面板', detailType: 'TABPAGE', name: 'tabpage1', visible: true, isShowCaption: true, form: this })
, 
        formitem2: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem2', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem3: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem3', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        tabpage2: new FormTabPageModel({ caption: '分页面板2', detailType: 'TABPAGE', name: 'tabpage2', visible: true, isShowCaption: true, form: this })
, 
        tabpanel1: new FormTabPanelModel({ caption: '', detailType: 'TABPANEL', name: 'tabpanel1', visible: true, isShowCaption: false, form: this, tabPages: [{ name: 'tabpage1', index: 0, visible: true }, { name: 'tabpage2', index: 1, visible: true }] })
, 
        grouppanel1: new FormGroupPanelModel({ caption: '分组面板', detailType: 'GROUPPANEL', name: 'grouppanel1', visible: true, isShowCaption: true, form: this, uiActionGroup: { caption: '', extractMode: 'ITEM', details: [] } })
, 
        button1: new FormButtonModel({ caption: '表单按钮', detailType: 'BUTTON', name: 'button1', visible: true, isShowCaption: true, form: this })
, 
        rawitem1: new FormRowItemModel({ caption: '', detailType: 'RAWITEM', name: 'rawitem1', visible: true, isShowCaption: true, form: this })
, 
        iframe1: new FormIFrameModel({ caption: '', detailType: 'IFRAME', name: 'iframe1', visible: true, isShowCaption: true, form: this })
, 
        grouppanel2: new FormGroupPanelModel({ caption: '分组面板2', detailType: 'GROUPPANEL', name: 'grouppanel2', visible: true, isShowCaption: true, form: this, uiActionGroup: { caption: '', extractMode: 'ITEM', details: [] } })
, 
        formitem4: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem4', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem5: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem5', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem6: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem6', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem7: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem7', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem8: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem8', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem9: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem9', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem10: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem10', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem11: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem11', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        grouppanel3: new FormGroupPanelModel({ caption: '栅格布局', detailType: 'GROUPPANEL', name: 'grouppanel3', visible: true, isShowCaption: true, form: this, uiActionGroup: { caption: '', extractMode: 'ITEM', details: [] } })
, 
        formitem12: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem12', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem13: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem13', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        formitem14: new FormItemModel({ caption: '表单项', detailType: 'FORMITEM', name: 'formitem14', visible: true, isShowCaption: true, form: this, disabled: false, enableCond: 3 })
, 
        grouppanel4: new FormGroupPanelModel({ caption: 'Flex布局', detailType: 'GROUPPANEL', name: 'grouppanel4', visible: true, isShowCaption: true, form: this, uiActionGroup: { caption: '', extractMode: 'ITEM', details: [] } })
, 
        formpage1: new FormPageModel({ caption: '表单分页', detailType: 'FORMPAGE', name: 'formpage1', visible: true, isShowCaption: true, form: this })
, 
    };

    /**
     * 生命周期
     *
     * @memberof EditForm
     */
    ngOnInit(): void {
        this.formLogic('', null, null);
        this.formDataChange.pipe(
			debounceTime(300),
            distinctUntilChanged()
		).subscribe(data => {
			this.dataChangeEvent.emit(JSON.parse(data.data));
			const field = data.field;
			if (field) {
				this.formLogic(field.name, field.value, field.oldVal);
				this.validateFormItem(field.name);
			}
		});
    }

    /**
     * 表单按钮事件触发
     *
     * @param {*} $event
     * @memberof EditForm
     */
    public button1_click($event: any) {

    }

	/**
	 * 表单项值变化事件
	 *
	 * @param {{name: string, value: any}} {name, value}
	 * @memberof EditForm
	 */
	public onFormDataChange(name: string, value: any): void {
		if (Object.is(this.data[name], value)) {
            return;
        }
		let oldVal: any = this.data[name];
		this.data[name] = value;
		this.formDataChange.next({ data: JSON.stringify(this.data), field: { name: name, value: value, oldVal: oldVal } });
	}

	/**
	 * 校验表单项
	 *
	 * @param {string} name
	 * @memberof EditForm
	 */
	public validateFormItem(name: string) {
		if (!name) {
			return;
		}
		const rule: any[] = this.rules[name];
		let message = '';
        let status = '';
        const val: any = this.data[name];
        rule.every((rule: any) => {
            if (rule.required && (!val || Object.is(val, '')) ) {
                message = rule.message;
                status = 'error';
                return false;
            } else if (val && rule.pattern && !rule.pattern.test(val)) {
                message = rule.message;
                status = 'error';
                return false;
            }
            return true;
        });
		this.detailsModel[name].error = message;
		this.detailsModel[name].validateStatus = status;
	}

    /**
     * 获取代码表集合
     *
     * @param {string} name
     * @memberof EditForm
     */
    public getCodeList(name: string) {
        if (!name || !this.codelistModel[name]) {
            return [];
        }
        return this.codelistModel[name];
    }

    /**
     * 表单逻辑
     *
     * @param {string} name
     * @param {*} newVal
     * @param {*} oldVal
     * @memberof EditForm
     */
    public formLogic(name: string, newVal: any, oldVal: any) {
                


























    }
}